from typing import Any

from django_filters import filterset as filterset

FILTER_FOR_DBFIELD_DEFAULTS: Any

class FilterSet(filterset.FilterSet):
    FILTER_DEFAULTS: Any = ...
    @property
    def form(self): ...



from typing import Any, Optional

from .exceptions import FieldLookupError as FieldLookupError

def deprecate(msg: Any, level_modifier: int = ...) -> None: ...

class MigrationNotice(DeprecationWarning):
    url: str = ...
    def __init__(self, message: Any) -> None: ...

class RenameAttributesBase(type):
    renamed_attributes: Any = ...
    def __new__(metacls: Any, name: Any, bases: Any, attrs: Any): ...
    def get_name(metacls: Any, name: Any): ...
    def __getattr__(metacls: Any, name: Any): ...
    def __setattr__(metacls: Any, name: Any, value: Any): ...

def try_dbfield(fn: Any, field_class: Any): ...
def get_all_model_fields(model: Any): ...
def get_model_field(model: Any, field_name: Any): ...
def get_field_parts(model: Any, field_name: Any): ...
def resolve_field(model_field: Any, lookup_expr: Any): ...
def handle_timezone(value: Any, is_dst: Optional[Any] = ...): ...
def verbose_field_name(model: Any, field_name: Any): ...
def verbose_lookup_expr(lookup_expr: Any): ...
def label_for_filter(
    model: Any, field_name: Any, lookup_expr: Any, exclude: bool = ...
): ...
def translate_validation(error_dict: Any): ...



from typing import Any, Dict, Optional, Tuple

from django import forms
from django.forms.renderers import BaseRenderer
from django.utils.safestring import SafeText

_OptAttrs = Dict[str, Any]

class LinkWidget(forms.Widget):
    choices: Any = ...
    def __init__(self, attrs: Optional[Any] = ..., choices: Any = ...) -> None: ...
    data: Any = ...
    def value_from_datadict(self, data: Any, files: Any, name: Any): ...
    def render(  # type: ignore[override]
        self,
        name: str,
        value: Any,
        attrs: Optional[_OptAttrs] = ...,
        choices: Tuple = ...,
        renderer: Optional[BaseRenderer] = ...,
    ) -> SafeText: ...
    def render_options(self, choices: Any, selected_choices: Any, name: Any): ...
    def render_option(
        self, name: Any, selected_choices: Any, option_value: Any, option_label: Any
    ): ...
    def option_string(self): ...

class SuffixedMultiWidget(forms.MultiWidget):
    suffixes: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def suffixed(self, name: Any, suffix: Any): ...
    def get_context(self, name: Any, value: Any, attrs: Any): ...
    def value_from_datadict(self, data: Any, files: Any, name: Any): ...
    def value_omitted_from_data(self, data: Any, files: Any, name: Any): ...
    def replace_name(self, output: Any, index: Any): ...
    def decompress(self, value: Any): ...

class RangeWidget(SuffixedMultiWidget):
    template_name: str = ...
    suffixes: Any = ...
    def __init__(self, attrs: Optional[Any] = ...) -> None: ...
    def decompress(self, value: Any): ...

class DateRangeWidget(RangeWidget):
    suffixes: Any = ...

class LookupChoiceWidget(SuffixedMultiWidget):
    suffixes: Any = ...
    def decompress(self, value: Any): ...

class BooleanWidget(forms.Select):
    def __init__(self, attrs: Optional[Any] = ...) -> None: ...
    def render(
        self,
        name: Any,
        value: Any,
        attrs: Optional[Any] = ...,
        renderer: Optional[Any] = ...,
    ): ...
    def value_from_datadict(self, data: Any, files: Any, name: Any): ...

class BaseCSVWidget(forms.Widget):
    def value_from_datadict(self, data: Any, files: Any, name: Any): ...
    def render(
        self,
        name: Any,
        value: Any,
        attrs: Optional[Any] = ...,
        renderer: Optional[Any] = ...,
    ): ...

class CSVWidget(BaseCSVWidget, forms.TextInput): ...

class QueryArrayWidget(BaseCSVWidget, forms.TextInput):
    def value_from_datadict(self, data: Any, files: Any, name: Any): ...



from typing import Any, Optional

from django.utils.deprecation import RenameMethodsBase

from .. import compat as compat
from .. import utils as utils
from . import filters as filters
from . import filterset as filterset

class RenameAttributes(utils.RenameAttributesBase, RenameMethodsBase):
    renamed_attributes: Any = ...
    renamed_methods: Any = ...

class DjangoFilterBackend(metaclass=RenameAttributes):
    filterset_base: Any = ...
    raise_exception: bool = ...
    @property
    def template(self): ...
    def get_filterset(self, request: Any, queryset: Any, view: Any): ...
    def get_filterset_class(self, view: Any, queryset: Optional[Any] = ...): ...
    def get_filterset_kwargs(self, request: Any, queryset: Any, view: Any): ...
    def filter_queryset(self, request: Any, queryset: Any, view: Any): ...
    def to_html(self, request: Any, queryset: Any, view: Any): ...
    def get_coreschema_field(self, field: Any): ...
    def get_schema_fields(self, view: Any): ...
    def get_schema_operation_parameters(self, view: Any): ...



from .backends import DjangoFilterBackend as DjangoFilterBackend
from .filters import *
from .filterset import FilterSet as FilterSet

__all__ = [
    "DjangoFilterBackend",
    "FilterSet",
    "BooleanFilter",
    "AllValuesFilter",
    "AllValuesMultipleFilter",
    "BaseCSVFilter",
    "BaseInFilter",
    "BaseRangeFilter",
    "CharFilter",
    "ChoiceFilter",
    "DateFilter",
    "DateFromToRangeFilter",
    "DateRangeFilter",
    "DateTimeFilter",
    "DateTimeFromToRangeFilter",
    "DurationFilter",
    "Filter",
    "IsoDateTimeFilter",
    "IsoDateTimeFromToRangeFilter",
    "LookupChoiceFilter",
    "ModelChoiceFilter",
    "ModelMultipleChoiceFilter",
    "MultipleChoiceFilter",
    "NumberFilter",
    "NumericRangeFilter",
    "OrderingFilter",
    "RangeFilter",
    "TimeFilter",
    "TimeRangeFilter",
    "TypedChoiceFilter",
    "TypedMultipleChoiceFilter",
    "UUIDFilter",
]



from typing import Any, Optional

from .conf import settings
from .constants import ALL_FIELDS
from .filters import (
    BaseInFilter,
    BaseRangeFilter,
    BooleanFilter,
    CharFilter,
    ChoiceFilter,
    DateFilter,
    DateTimeFilter,
    DurationFilter,
    Filter,
    ModelChoiceFilter,
    ModelMultipleChoiceFilter,
    NumberFilter,
    TimeFilter,
    UUIDFilter,
)
from .utils import get_all_model_fields, get_model_field, resolve_field, try_dbfield

def remote_queryset(field: Any): ...

class FilterSetOptions:
    model: Any = ...
    fields: Any = ...
    exclude: Any = ...
    filter_overrides: Any = ...
    form: Any = ...
    def __init__(self, options: Optional[Any] = ...) -> None: ...

class FilterSetMetaclass(type):
    def __new__(cls, name: Any, bases: Any, attrs: Any): ...
    @classmethod
    def get_declared_filters(cls, bases: Any, attrs: Any): ...

FILTER_FOR_DBFIELD_DEFAULTS: Any

class BaseFilterSet:
    FILTER_DEFAULTS: Any = ...
    is_bound: Any = ...
    data: Any = ...
    queryset: Any = ...
    request: Any = ...
    form_prefix: Any = ...
    filters: Any = ...
    def __init__(
        self,
        data: Optional[Any] = ...,
        queryset: Optional[Any] = ...,
        *,
        request: Optional[Any] = ...,
        prefix: Optional[Any] = ...
    ) -> None: ...
    def is_valid(self): ...
    @property
    def errors(self): ...
    def filter_queryset(self, queryset: Any): ...
    @property
    def qs(self): ...
    def get_form_class(self): ...
    @property
    def form(self): ...
    @classmethod
    def get_fields(cls): ...
    @classmethod
    def get_filter_name(cls, field_name: Any, lookup_expr: Any): ...
    @classmethod
    def get_filters(cls): ...
    @classmethod
    def filter_for_field(cls, field: Any, field_name: Any, lookup_expr: str = ...): ...
    @classmethod
    def filter_for_lookup(cls, field: Any, lookup_type: Any): ...

class FilterSet(BaseFilterSet, metaclass=FilterSetMetaclass): ...

def filterset_factory(model: Any, fields: Any = ...): ...



from typing import Any

from .utils import deprecate as deprecate

DEFAULTS: Any
DEPRECATED_SETTINGS: Any

def is_callable(value: Any): ...

class Settings:
    def __getattr__(self, name: Any): ...
    def get_setting(self, setting: Any): ...
    def change_setting(
        self, setting: Any, value: Any, enter: Any, **kwargs: Any
    ) -> None: ...

settings: Any



from typing import Any, Optional

from django import forms

from .conf import settings as settings
from .constants import EMPTY_VALUES as EMPTY_VALUES
from .utils import handle_timezone as handle_timezone
from .widgets import BaseCSVWidget as BaseCSVWidget
from .widgets import CSVWidget as CSVWidget
from .widgets import DateRangeWidget as DateRangeWidget
from .widgets import LookupChoiceWidget as LookupChoiceWidget
from .widgets import RangeWidget as RangeWidget

class RangeField(forms.MultiValueField):
    widget: Any = ...
    def __init__(
        self, fields: Optional[Any] = ..., *args: Any, **kwargs: Any
    ) -> None: ...
    def compress(self, data_list: Any): ...

class DateRangeField(RangeField):
    widget: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def compress(self, data_list: Any): ...

class DateTimeRangeField(RangeField):
    widget: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class IsoDateTimeRangeField(RangeField):
    widget: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class TimeRangeField(RangeField):
    widget: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class Lookup:
    def __new__(cls, value: Any, lookup_expr: Any): ...

class LookupChoiceField(forms.MultiValueField):
    default_error_messages: Any = ...
    def __init__(
        self, field: Any, lookup_choices: Any, *args: Any, **kwargs: Any
    ) -> None: ...
    def compress(self, data_list: Any): ...

class IsoDateTimeField(forms.DateTimeField):
    ISO_8601: str = ...
    input_formats: Any = ...
    def strptime(self, value: Any, format: Any): ...

class BaseCSVField(forms.Field):
    base_widget_class: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def clean(self, value: Any): ...

class BaseRangeField(BaseCSVField):
    widget: Any = ...
    default_error_messages: Any = ...
    def clean(self, value: Any): ...

class ChoiceIterator:
    field: Any = ...
    choices: Any = ...
    def __init__(self, field: Any, choices: Any) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self): ...

class ModelChoiceIterator(forms.models.ModelChoiceIterator):
    def __iter__(self) -> Any: ...
    def __len__(self): ...

class ChoiceIteratorMixin:
    null_label: Any = ...
    null_value: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    choices: Any = ...

class ChoiceField(ChoiceIteratorMixin, forms.ChoiceField):
    iterator: Any = ...
    empty_label: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class MultipleChoiceField(ChoiceIteratorMixin, forms.MultipleChoiceField):
    iterator: Any = ...
    empty_label: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class ModelChoiceField(ChoiceIteratorMixin, forms.ModelChoiceField):
    iterator: Any = ...
    def to_python(self, value: Any): ...

class ModelMultipleChoiceField(ChoiceIteratorMixin, forms.ModelMultipleChoiceField):
    iterator: Any = ...



from typing import Any, Optional

from django.views.generic import View
from django.views.generic.list import (
    MultipleObjectMixin,
    MultipleObjectTemplateResponseMixin,
)

from .constants import ALL_FIELDS as ALL_FIELDS
from .filterset import filterset_factory as filterset_factory
from .utils import MigrationNotice as MigrationNotice
from .utils import RenameAttributesBase as RenameAttributesBase

class FilterMixinRenames(RenameAttributesBase):
    renamed_attributes: Any = ...

class FilterMixin(metaclass=FilterMixinRenames):
    filterset_class: Any = ...
    filterset_fields: Any = ...
    strict: bool = ...
    def get_filterset_class(self): ...
    def get_filterset(self, filterset_class: Any): ...
    def get_filterset_kwargs(self, filterset_class: Any): ...
    def get_strict(self): ...

class BaseFilterView(FilterMixin, MultipleObjectMixin, View):
    filterset: Any = ...
    object_list: Any = ...
    def get(self, request: Any, *args: Any, **kwargs: Any): ...

class FilterView(MultipleObjectTemplateResponseMixin, BaseFilterView):
    template_name_suffix: str = ...

def object_filter(
    request: Any,
    model: Optional[Any] = ...,
    queryset: Optional[Any] = ...,
    template_name: Optional[Any] = ...,
    extra_context: Optional[Any] = ...,
    context_processors: Optional[Any] = ...,
    filter_class: Optional[Any] = ...,
): ...



import os
from pathlib import Path

from setuptools import setup

name = "django-filter-stubs"


def find_stub_files(name):
    result = []
    for root, dirs, files in os.walk(name):
        for f in files:
            if f.endswith(".pyi"):
                if os.path.sep in root:
                    sub_root = root.split(os.path.sep, 1)[-1]
                    f = os.path.join(sub_root, f)
                result.append(f)
    return result


long_description = (Path(__file__).parent / "README.md").read_text()


setup(
    name="django-filter-stubs",
    version="0.1.3",
    description="PEP-484 stubs for django-filter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/DavisRayM/django-filter-stubs",
    author="Davis Raymond Muro",
    author_email="davisraymondmuro@outlook.com",
    packages=['django_filters-stubs'],
    package_data={"django_filters-stubs": find_stub_files('django_filters-stubs')},
    install_requires=[
        "mypy>=0.750",
        "django-stubs>=1.3.0",
        "djangorestframework-stubs>=0.4.0",
        "typing-extensions>=3.7.4",
    ],
    classifiers=[
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
    ],
)



from typing import Any, Callable, Optional

class Filter:
    creation_counter: int = ...
    field_class: Any = ...
    field_name: Any = ...
    lookup_expr: Any = ...
    distinct: Any = ...
    exclude: Any = ...
    extra: Any = ...
    def __init__(
        self,
        field_name: Optional[Any] = ...,
        lookup_expr: str = ...,
        *,
        label: Optional[Any] = ...,
        method: Optional[Any] = ...,
        distinct: bool = ...,
        exclude: bool = ...,
        **kwargs: Any
    ) -> None: ...
    def get_method(self, qs: Any): ...
    method: Callable = ...
    label: Any = ...
    @property
    def field(self): ...
    def filter(self, qs: Any, value: Any): ...

class CharFilter(Filter):
    field_class: Any = ...

class BooleanFilter(Filter):
    field_class: Any = ...

class ChoiceFilter(Filter):
    field_class: Any = ...
    null_value: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def filter(self, qs: Any, value: Any): ...

class TypedChoiceFilter(Filter):
    field_class: Any = ...

class UUIDFilter(Filter):
    field_class: Any = ...

class MultipleChoiceFilter(Filter):
    field_class: Any = ...
    always_filter: bool = ...
    conjoined: Any = ...
    null_value: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def is_noop(self, qs: Any, value: Any): ...
    def filter(self, qs: Any, value: Any): ...
    def get_filter_predicate(self, v: Any): ...

class TypedMultipleChoiceFilter(MultipleChoiceFilter):
    field_class: Any = ...

class DateFilter(Filter):
    field_class: Any = ...

class DateTimeFilter(Filter):
    field_class: Any = ...

class IsoDateTimeFilter(DateTimeFilter):
    field_class: Any = ...

class TimeFilter(Filter):
    field_class: Any = ...

class DurationFilter(Filter):
    field_class: Any = ...

class QuerySetRequestMixin:
    queryset: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def get_request(self): ...
    def get_queryset(self, request: Any): ...
    @property
    def field(self): ...

class ModelChoiceFilter(QuerySetRequestMixin, ChoiceFilter):
    field_class: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class ModelMultipleChoiceFilter(QuerySetRequestMixin, MultipleChoiceFilter):
    field_class: Any = ...

class NumberFilter(Filter):
    field_class: Any = ...

class NumericRangeFilter(Filter):
    field_class: Any = ...
    lookup_expr: str = ...
    def filter(self, qs: Any, value: Any): ...

class RangeFilter(Filter):
    field_class: Any = ...
    lookup_expr: str = ...
    def filter(self, qs: Any, value: Any): ...

class DateRangeFilter(ChoiceFilter):
    choices: Any = ...
    filters: Any = ...
    def __init__(
        self,
        choices: Optional[Any] = ...,
        filters: Optional[Any] = ...,
        *args: Any,
        **kwargs: Any
    ) -> None: ...
    def filter(self, qs: Any, value: Any): ...

class DateFromToRangeFilter(RangeFilter):
    field_class: Any = ...

class DateTimeFromToRangeFilter(RangeFilter):
    field_class: Any = ...

class IsoDateTimeFromToRangeFilter(RangeFilter):
    field_class: Any = ...

class TimeRangeFilter(RangeFilter):
    field_class: Any = ...

class AllValuesFilter(ChoiceFilter):
    @property
    def field(self): ...

class AllValuesMultipleFilter(MultipleChoiceFilter):
    @property
    def field(self): ...

class BaseCSVFilter(Filter):
    base_field_class: Any = ...
    field_class: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class BaseInFilter(BaseCSVFilter):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class BaseRangeFilter(BaseCSVFilter):
    base_field_class: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class LookupChoiceFilter(Filter):
    field_class: Any = ...
    outer_class: Any = ...
    empty_label: Any = ...
    lookup_choices: Any = ...
    def __init__(
        self,
        field_name: Optional[Any] = ...,
        lookup_choices: Optional[Any] = ...,
        field_class: Optional[Any] = ...,
        **kwargs: Any
    ) -> None: ...
    @classmethod
    def normalize_lookup(cls, lookup: Any): ...
    def get_lookup_choices(self): ...
    @property
    def field(self): ...
    lookup_expr: Any = ...
    def filter(self, qs: Any, lookup: Any): ...

class OrderingFilter(BaseCSVFilter, ChoiceFilter):
    descending_fmt: Any = ...
    param_map: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def get_ordering_value(self, param: Any): ...
    def filter(self, qs: Any, value: Any): ...
    @classmethod
    def normalize_fields(cls, fields: Any): ...
    def build_choices(self, fields: Any, labels: Any): ...

class FilterMethod:
    f: Any = ...
    def __init__(self, filter_instance: Any) -> None: ...
    def __call__(self, qs: Any, value: Any): ...
    @property
    def method(self): ...
